"use client";

import { Button } from "@/components/ui/button";
import {
	Dialog,
	DialogClose,
	DialogContent,
	DialogDescription,
	DialogFooter,
	DialogHeader,
	DialogTitle,
	DialogTrigger,
} from "@/components/ui/dialog";
import { useEffect, useMemo, useState, useTransition } from "react";

// Import FormBuilder components
import FormBuilder from "@/components/FormBuilder/form-builder";
import AutocompleteField from "@/components/FormBuilder/FormFields/autocomplete-field";
import InputField from "@/components/FormBuilder/FormFields/input-field";
import useFormBuilder from "@/components/FormBuilder/hooks/use-form-builder";
import prepareFields from "@/components/FormBuilder/utils/prepare-fields";
import Yup from "@/components/FormBuilder/utils/yup";
import { Map } from "@/components/ui/map";
import { fetchArea, fetchCity } from "@/servers/actions/city-area-action";
import { Loader2 } from "lucide-react";
import { useLocale } from "next-intl";
import { toast } from "sonner";

interface AddressDialogProps {
	onSave?: (data: any) => void;
	initialData?: {
		city_id?: string;
		area_id?: string;
		address?: string;
		additional_description?: string;
		latitude?: number | null;
		longitude?: number | null;
	};
	triggerButton?: React.ReactNode;
	disabledFields?: string[];
}

export default function AddressDialog({ onSave, initialData, triggerButton, disabledFields = [] }: AddressDialogProps) {

	const locale = useLocale();
	const [isPending, startTransition] = useTransition();
	const [isOpen, setIsOpen] = useState(false);
	const [selectedLocation, setSelectedLocation] = useState<{ lat: number; lng: number } | null>(
		initialData?.latitude && initialData?.longitude
			? { lat: initialData.latitude, lng: initialData.longitude }
			: { lat: 24.7136, lng: 46.6753 } // Default coordinates for Riyadh, Saudi Arabia
	);
	const [selectedCity, setSelectedCity] = useState<string | null>(initialData?.city_id || null);
	const [cities, setCities] = useState<{ value: string; label: string }[]>([]);
	const [areas, setAreas] = useState<{ value: string; label: string }[]>([]);
	const [isLoadingAreas, setIsLoadingAreas] = useState(false);
	const [locationError, setLocationError] = useState<string | null>(null);
	const [cityError, setCityError] = useState<string | null>(null);


	// Fetch cities on component mount
	useEffect(() => {
		const loadCities = async () => {
			try {
				const citiesData = (await fetchCity())?.data;
				console.log("citiesData: ", citiesData);
				const formattedCities = citiesData?.map((city: any) => ({
					value: city.id.toString(),
					label: locale === "ar" ? city.name_ar : city.name_en,
				}));
				setCities(formattedCities);
			} catch (error) {
				console.error("Error fetching cities:", error);
				toast.error("حدث خطأ أثناء تحميل المدن");
			}
		};
		loadCities();
	}, []);

	// Fetch areas when city changes
	useEffect(() => {
		const loadAreas = async () => {
			if (!selectedCity) {
				setAreas([]);
				return;
			}

			setIsLoadingAreas(true);
			try {
				const areasData = (await fetchArea(selectedCity))?.data;
				const formattedAreas = areasData?.map((area: any) => ({
					value: area.id.toString(),
					label: locale === "ar" ? area.name_ar : area.name_en,
					delivery_price: area?.delivery_price,
				}));
				setAreas(formattedAreas);
			} catch (error) {
				console.error("Error fetching areas:", error);
				toast.error("حدث خطأ أثناء تحميل المناطق");
			} finally {
				setIsLoadingAreas(false);
			}
		};
		loadAreas();
	}, [selectedCity]);


	// Define fields for the address dialog using useFormBuilder
	const addressFields = useMemo(
		() =>
			prepareFields({
				city_id: {
					yup: Yup.string().required("المدينة مطلوبة"),
					type: "autocomplete",
					component: AutocompleteField,
					componentProps: ({ }) => ({
						label: "المدينة *",
						placeholder: "اختر المدينة",
						containerClassName: "col-span-6",
						options: cities,
						onChange: (value: string) => {
							setSelectedCity(value);
							setCityError(null); // Clear city error when a city is selected
							// Reset area when city changes
							AddressFormHooks.setValue("area_id", "");
						},
						error: cityError,
						disabled: disabledFields.includes("city"),
					}),
				},
				area_id: {
					yup: Yup.string().required("المنطقة مطلوبة"),
					type: "autocomplete",
					component: AutocompleteField,
					componentProps: ({ }) => ({
						label: "المنطقة *",
						placeholder: "اختر المنطقة",
						containerClassName: "col-span-6",
						options: areas,
						disabled: !selectedCity || isLoadingAreas || disabledFields.includes("area"),
					}),
				},
				address: {
					yup: Yup.string().required("العنوان مطلوب"),
					type: "input",
					component: InputField,
					componentProps: ({ }) => ({
						type: "text",
						label: "العنوان *",
						placeholder: "أدخل العنوان",
						containerClassName: "col-span-12",
						disabled: disabledFields.includes("address"),
					}),
				},
				additional_description: {
					yup: Yup.string().nullable(),
					type: "input",
					component: InputField,
					componentProps: ({ }) => ({
						label: "وصف إضافي",
						placeholder: "أدخل وصفاً إضافياً للعنوان (اختياري)",
						containerClassName: "col-span-12",
						rows: 3,
						disabled: disabledFields.includes("additional_description"),
					}),
				},
			}),
		[selectedCity, cities, areas, isLoadingAreas, cityError, disabledFields]
	);

	// Initialize form hooks for the address dialog
	const AddressFormHooks = useFormBuilder(addressFields, {
		defaultValues: {
			city_id: initialData?.city_id || "",
			area_id: initialData?.area_id || "",
			address: initialData?.address || "",
			additional_description: initialData?.additional_description || "",
		},
	});

	// Handle form submission for the address dialog
	async function onSubmitAddressInfo(data: any) {
		console.log("data: ", data);
		try {
			// Validate that city is selected
			if (!data.city_id) {
				setCityError("يرجى اختيار المدينة");
				toast.error("يرجى اختيار المدينة");
				return;
			}

			// Validate that location is selected
			if (!selectedLocation) {
				setLocationError("يرجى تحديد موقعك على الخريطة");
				toast.error("يرجى تحديد موقعك على الخريطة");
				return;
			}

			const _city = cities?.find(city => city.value === data?.city_id) as any;
			const _area = areas?.find(area => area.value === data?.area_id) as any;

			startTransition(async () => {
				console.log("Saving address data:", data);

				// Add location data to the form data
				const addressData = {
					...data,
					city_name: _city?.label,
					area_name: _area?.label,
					delivery_price: _area?.delivery_price,
					location_lat: selectedLocation?.lat,
					location_lng: selectedLocation?.lng,
				};

				// Call the onSave callback if provided
				if (onSave) {
					onSave(addressData);

				}

				// Show success message
				toast.success("تم حفظ عنوان التوصيل بنجاح", {
					style: {
						backgroundColor: "green",
						color: "white",
					},
				});

				// Close the dialog after saving
				setIsOpen(false);
			});
		} catch (error) {
			console.error("Form submission error:", error);
			toast.error("حدث خطأ أثناء حفظ العنوان");
		}
	}

	// Handle map selection
	const handleMapSelect = (lat: number, lng: number) => {
		setSelectedLocation({ lat, lng });
		setLocationError(null); // Clear error when location is selected
	};

	return (
		<Dialog open={isOpen} onOpenChange={setIsOpen}>
			<DialogTrigger asChild>
				{triggerButton || <Button className="w-full bg-gray-600 text-white">إضافة عنوان جديد</Button>}
			</DialogTrigger>
			<DialogContent className="sm:max-w-[700px] max-h-[700px] overflow-y-auto scrollbar-thin scrollbar-thumb-gray-400 scrollbar-track-gray-200" dir="rtl">
				<DialogHeader>
					<DialogTitle>إضافة عنوان توصيل</DialogTitle>
					<DialogDescription>أدخل تفاصيل عنوان التوصيل الخاص بك</DialogDescription>
				</DialogHeader>
				<form onSubmit={AddressFormHooks.handleSubmit(onSubmitAddressInfo)}>
					<div>
						<FormBuilder fields={addressFields} FormHooks={AddressFormHooks} />

						{/* Map for location selection */}
						<div className="mt-4 col-span-12">
							<label className="block text-sm font-medium mb-2">
								موقعك على الخريطة <span className="text-red-500">*</span>
							</label>
							<div className={`h-[400px] w-full rounded-md border ${locationError ? "border-red-500" : "border-none"}`}>
								<Map
									onLocationSelect={handleMapSelect}
									initialPosition={selectedLocation ? [selectedLocation.lat, selectedLocation.lng] : undefined}
								/>
							</div>
							{locationError && (
								<p className="text-xs text-red-500 mt-1">{locationError}</p>
							)}
							<p className="text-xs text-muted-foreground mt-1">
								انقر على الخريطة لتحديد موقعك بدقة
							</p>
						</div>
					</div>
					<DialogFooter className="flex justify-end gap-2">
						<Button type="submit" disabled={isPending}>
							{isPending ? <Loader2 className="mx-auto w-4 h-4 animate-spin" /> : "حفظ"}
						</Button>
						<DialogClose asChild><Button type="button" variant="outline">إلغاء</Button></DialogClose>
					</DialogFooter>
				</form>
			</DialogContent>
		</Dialog>
	);
}